local settings = core_settings_settings
local directory = "replays/"
local M = {}

local function getFileStream()
  if not be then return end
  return be:getFileStream()
end

local function setSaveCount(count)
  local previousValue = settings.getValue("torsionScenarioReplaySaver_SaveCount")
  settings.setValue("torsionScenarioReplaySaver_SaveCount",count)
  log('I', "Torsion's Scenario Replay Saver", "Slot Count set to " .. count)
  log('I', "Torsion's Scenario Replay Saver", "Slot Count setting saved")
  if previousValue ~= nil then
    if count < previousValue then
      -- delete all abandoned slots
      for i=previousValue,count+1,-1 do 
        local directory = "replays/"
        local filePaths = FS:findFiles(directory, "[TSRS-Slot_" .. string.format("%03d",i) .. "]*.rpl", -1, true, false)
        for _, file in ipairs(filePaths) do
          log('I', "Torsion's Scenario Replay Saver", "Deleting abandoned save slot " .. file)
          FS:removeFile(file)
        end
      end
    end
  end
end

local function setIncludeCountdown(value)
  if type(value) ~= "boolean" then
    log('E', "Torsion's Scenario Replay Saver", "Expected boolean but got " .. type(value) .. ".  Ignoring.")
    value = true
  end 

  settings.setValue("torsionScenarioReplaySaver_IncludeCountdown",value)
  log('I', "Torsion's Scenario Replay Saver", "Include Countdown set to '" .. tostring(value) .. "'")
  log('I', "Torsion's Scenario Replay Saver", "Include Countdown setting saved")
end

local function shuffleFilesDown()
  local numSavesPreference = settings.getValue("torsionScenarioReplaySaver_SaveCount")
  if numSavesPreference == nil then 
    numSavesPreference = 3
    setSaveCount(3)
  end

  -- delete oldest save
  local filePaths = FS:findFiles(directory, "[TSRS-Slot_" .. string.format("%03d",numSavesPreference) .. "]*.rpl", -1, true, false)
  for _, file in ipairs(filePaths) do
    log('I', "Torsion's Scenario Replay Saver", "Deleting " .. file)
    FS:removeFile(file)
  end
  
  -- rename all other saves to +1
  for i=numSavesPreference,1,-1 do 
    --print(i)
    local filePaths = FS:findFiles(directory, "[TSRS-Slot_" .. string.format("%03d",i) .. "]*.rpl", -1, true, false)
    for _, file in ipairs(filePaths) do
      local newConstructedName = string.sub(file,1,20) .. string.format("%03d",i+1) .. string.sub(file,24,-1) -- 
      log('I', "Torsion's Scenario Replay Saver", "Renaming " .. file .. " to " .. newConstructedName)
      FS:renameFile(file, newConstructedName)
      FS:removeFile(file)
    end
  end
  
end

local function startRecording()
  local pre = "[TSRS-Slot_001] "
  local stream = getFileStream()
  if not stream then return end
  
  --core_replay.cancelRecording()
  stream:stop()
  shuffleFilesDown()

  local date = os.date("%Y-%m-%d_%H-%M-%S")
  local map = getMissionFilename():match('levels/([%w|_|%-|%s]+)/')
  if map == nil then
    log("E", "", "Cannot start recording replay. Map filename: "..dumps(getMissionFilename()))
  else
    local filename = "replays/".. pre ..date.." "..map..".rpl"
    log("D","",'record to: '..filename)
    ui_message("replay.startRecording", 5, "replay", "local_movies")
    stream:record(filename)
  end
    
	log('I', "Torsion's Scenario Replay Saver", "Starting Recording...")
end

local function stopRecording()  
  local state = core_replay.getState()
  local stream = getFileStream()
  if not stream then return end
	
--  log("D","", 'Stopping from state: '..state);
  
  if state == 'recording' then
    ui_message("replay.stopRecording", 5, "replay", "local_movies")
    log('I', "Torsion's Scenario Replay Saver", "Stopping Recording...")
    stream:stop()
  end
end

local function onCountdownStarted()
  local includeCountdown = settings.getValue("torsionScenarioReplaySaver_IncludeCountdown")
  if includeCountdown ~= false then
    startRecording()
  end  
end

local function onRaceStart()
  local includeCountdown = settings.getValue("torsionScenarioReplaySaver_IncludeCountdown")
  if includeCountdown == false then
    startRecording()
  end  
end

M.onRaceStart = onRaceStart
M.onCountdownStarted = onCountdownStarted
M.onScenarioRestarted = stopRecording
M.onScenarioLoaded = stopRecording
M.onScenarioChange = stopRecording
M.onClientEndMission = stopRecording

M.setSaveCount = setSaveCount
M.setIncludeCountdown = setIncludeCountdown

--M.shuffleFilesDown = shuffleFilesDown

return M